@@ -28,14 +28,14 @@ RT_C_DECLS_END
 
 #ifdef DEBUG_misha
 /* specifies whether the vboxVDbgBreakF should break in the debugger
- * windbg seems to have some issues when there is a lot ( >~50) of sw breakpoints defined
+ * windbg seems to have some issues when there is a lot ( >32) of sw breakpoints defined
  * to simplify things we just insert breaks for the case of intensive debugging WDDM driver*/
 #ifndef VBOX_WDDM_WIN8
-int g_bVBoxVDbgBreakF = false;
-int g_bVBoxVDbgBreakFv = false;
+int g_bVBoxVDbgBreakF = 0;
+int g_bVBoxVDbgBreakFv = 0;
 #else
-int g_bVBoxVDbgBreakF = false;
-int g_bVBoxVDbgBreakFv = false;
+int g_bVBoxVDbgBreakF = 0;
+int g_bVBoxVDbgBreakFv = 0;
 #endif
 #endif
 
@@ -51,29 +51,31 @@ vboxWinVersion_t VBoxQueryWinVersion()
 {
     ULONG major, minor, build;
     BOOLEAN checkedBuild;
-    static vboxWinVersion_t s_WinVersion = UNKNOWN_WINVERSION;
+    static vboxWinVersion_t s_WinVersion = WINVERSION_UNKNOWN;
 
-    if (s_WinVersion != UNKNOWN_WINVERSION)
+    if (s_WinVersion != WINVERSION_UNKNOWN)
         return s_WinVersion;
 
     checkedBuild = PsGetVersion(&major, &minor, &build, NULL);
     LOG(("running on version %d.%d, build %d(checked=%d)", major, minor, build, (int)checkedBuild));
 
-    if(major == 6)
+    if (major == 6)
     {
-        if (minor == 3)
-            s_WinVersion = WIN81;
+        if (minor >= 4)
+            s_WinVersion = WINVERSION_10;
+        else if (minor == 3)
+            s_WinVersion = WINVERSION_81;
         else if (minor == 2)
-            s_WinVersion = WIN8;
+            s_WinVersion = WINVERSION_8;
         else if (minor == 1)
-            s_WinVersion = WIN7;
+            s_WinVersion = WINVERSION_7;
         else if (minor == 0)
-            s_WinVersion = WINVISTA; /* Or Windows Server 2008. */
+            s_WinVersion = WINVERSION_VISTA; /* Or Windows Server 2008. */
     }
     else if (major == 5)
-        s_WinVersion = (minor>=1) ? WINXP:WIN2K;
+        s_WinVersion = (minor>=1) ? WINVERSION_XP: WINVERSION_2K;
     else if (major == 4)
-        s_WinVersion = WINNT4;
+        s_WinVersion = WINVERSION_NT4;
     else
         WARN(("NT4 required!"));
     return s_WinVersion;
