@@ -179,7 +179,7 @@ PVOID vboxWddmHTableIterRemoveCur(PVBOXWDDM_HTABLE_ITERATOR pIter)
     }
     return NULL;
 }
-
+#ifdef VBOX_WITH_CROGL
 PVBOXWDDM_SWAPCHAIN vboxWddmSwapchainCreate(UINT w, UINT h)
 {
     PVBOXWDDM_SWAPCHAIN pSwapchain = (PVBOXWDDM_SWAPCHAIN)vboxWddmMemAllocZero(sizeof (VBOXWDDM_SWAPCHAIN));
@@ -571,7 +571,7 @@ VOID vboxWddmSwapchainCtxTerm(PVBOXMP_DEVEXT pDevExt, PVBOXWDDM_CONTEXT pContext
     vboxWddmSwapchainCtxDestroyAll(pDevExt, pContext);
     vboxWddmHTableDestroy(&pContext->Swapchains);
 }
-
+#endif
 NTSTATUS vboxWddmRegQueryDrvKeyName(PVBOXMP_DEVEXT pDevExt, ULONG cbBuf, PWCHAR pBuf, PULONG pcbResult)
 {
     WCHAR fallBackBuf[2];
@@ -618,14 +618,14 @@ NTSTATUS vboxWddmRegQueryDisplaySettingsKeyName(PVBOXMP_DEVEXT pDevExt, D3DDDI_V
         return STATUS_UNSUCCESSFUL;
 
     vboxWinVersion_t ver = VBoxQueryWinVersion();
-    if (ver == WINVISTA)
+    if (ver == WINVERSION_VISTA)
     {
         pKeyPrefix = VBOXWDDM_REG_DISPLAYSETTINGSKEY_PREFIX_VISTA;
         cbKeyPrefix = sizeof (VBOXWDDM_REG_DISPLAYSETTINGSKEY_PREFIX_VISTA);
     }
     else
     {
-        Assert(ver == WIN7 || ver == WIN8 || ver == WIN81);
+        Assert(ver > WINVERSION_VISTA);
         pKeyPrefix = VBOXWDDM_REG_DISPLAYSETTINGSKEY_PREFIX_WIN7;
         cbKeyPrefix = sizeof (VBOXWDDM_REG_DISPLAYSETTINGSKEY_PREFIX_WIN7);
     }
@@ -706,7 +706,7 @@ NTSTATUS vboxWddmRegQueryVideoGuidString(PVBOXMP_DEVEXT pDevExt, ULONG cbBuf, PW
         WARN(("failed to acquire the VideoID, falling back to the old impl"));
 
     Status = vboxWddmRegOpenKey(&hKey, VBOXWDDM_REG_DISPLAYSETTINGSVIDEOKEY, GENERIC_READ);
-    Assert(Status == STATUS_SUCCESS);
+    //Assert(Status == STATUS_SUCCESS);
     if (Status == STATUS_SUCCESS)
     {
         struct
@@ -733,7 +733,7 @@ NTSTATUS vboxWddmRegQueryVideoGuidString(PVBOXMP_DEVEXT pDevExt, ULONG cbBuf, PW
             pSubBuf += Buf.Name.NameLength/2;
             memcpy(pSubBuf, VBOXWDDM_REG_DISPLAYSETTINGSVIDEOKEY_SUBKEY, sizeof (VBOXWDDM_REG_DISPLAYSETTINGSVIDEOKEY_SUBKEY));
             Status = vboxWddmRegOpenKey(&hSubKey, KeyBuf, GENERIC_READ);
-            Assert(Status == STATUS_SUCCESS);
+            //Assert(Status == STATUS_SUCCESS);
             if (Status == STATUS_SUCCESS)
             {
                 struct
@@ -803,11 +803,11 @@ NTSTATUS vboxWddmRegOpenKey(OUT PHANDLE phKey, IN PWCHAR pName, IN ACCESS_MASK f
     return vboxWddmRegOpenKeyEx(phKey, NULL, pName, fAccess);
 }
 
-NTSTATUS vboxWddmRegOpenDisplaySettingsKey(IN PVBOXMP_DEVEXT pDeviceExtension, D3DDDI_VIDEO_PRESENT_SOURCE_ID VidPnSourceId, OUT PHANDLE phKey)
+NTSTATUS vboxWddmRegOpenDisplaySettingsKey(IN PVBOXMP_DEVEXT pDevExt, D3DDDI_VIDEO_PRESENT_SOURCE_ID VidPnSourceId, OUT PHANDLE phKey)
 {
     WCHAR Buf[512];
     ULONG cbBuf = sizeof(Buf);
-    NTSTATUS Status = vboxWddmRegQueryDisplaySettingsKeyName(pDeviceExtension, VidPnSourceId, cbBuf, Buf, &cbBuf);
+    NTSTATUS Status = vboxWddmRegQueryDisplaySettingsKeyName(pDevExt, VidPnSourceId, cbBuf, Buf, &cbBuf);
     Assert(Status == STATUS_SUCCESS);
     if (Status == STATUS_SUCCESS)
     {
@@ -850,12 +850,12 @@ NTSTATUS vboxWddmRegDisplaySettingsQueryRelY(HANDLE hKey, int * pResult)
     return Status;
 }
 
-NTSTATUS vboxWddmDisplaySettingsQueryPos(IN PVBOXMP_DEVEXT pDeviceExtension, D3DDDI_VIDEO_PRESENT_SOURCE_ID VidPnSourceId, POINT * pPos)
+NTSTATUS vboxWddmDisplaySettingsQueryPos(IN PVBOXMP_DEVEXT pDevExt, D3DDDI_VIDEO_PRESENT_SOURCE_ID VidPnSourceId, POINT * pPos)
 {
     Assert(KeGetCurrentIrql() == PASSIVE_LEVEL);
     HANDLE hKey;
-    NTSTATUS Status = vboxWddmRegOpenDisplaySettingsKey(pDeviceExtension, VidPnSourceId, &hKey);
-    Assert(Status == STATUS_SUCCESS);
+    NTSTATUS Status = vboxWddmRegOpenDisplaySettingsKey(pDevExt, VidPnSourceId, &hKey);
+    //Assert(Status == STATUS_SUCCESS);
     if (Status == STATUS_SUCCESS)
     {
         int x, y;
@@ -878,6 +878,27 @@ NTSTATUS vboxWddmDisplaySettingsQueryPos(IN PVBOXMP_DEVEXT pDeviceExtension, D3D
     return Status;
 }
 
+void vboxWddmDisplaySettingsCheckPos(IN PVBOXMP_DEVEXT pDevExt, D3DDDI_VIDEO_PRESENT_SOURCE_ID VidPnSourceId)
+{
+    POINT Pos = {0};
+    NTSTATUS Status = vboxWddmDisplaySettingsQueryPos(pDevExt, VidPnSourceId, &Pos);
+    if (!NT_SUCCESS(Status))
+    {
+        Log(("vboxWddmDisplaySettingsQueryPos failed %#x", Status));
+        return;
+    }
+
+    PVBOXWDDM_SOURCE pSource = &pDevExt->aSources[VidPnSourceId];
+
+    if (!memcmp(&pSource->VScreenPos, &Pos, sizeof (Pos)))
+        return;
+
+    pSource->VScreenPos = Pos;
+    pSource->u8SyncState &= ~VBOXWDDM_HGSYNC_F_SYNCED_DIMENSIONS;
+
+    vboxWddmGhDisplayCheckSetInfoFromSource(pDevExt, pSource);
+}
+
 NTSTATUS vboxWddmRegDrvFlagsSet(PVBOXMP_DEVEXT pDevExt, DWORD fVal)
 {
     HANDLE hKey = NULL;
@@ -1061,6 +1082,7 @@ typedef struct VBOXVIDEOCM_ALLOC_REF
     PKEVENT pSynchEvent;
     VBOXUHGSMI_BUFFER_TYPE_FLAGS fUhgsmiType;
     volatile uint32_t cRefs;
+    PVOID pvUm;
     MDL Mdl;
 } VBOXVIDEOCM_ALLOC_REF, *PVBOXVIDEOCM_ALLOC_REF;
 
@@ -1182,6 +1204,7 @@ NTSTATUS vboxVideoAMgrCtxAllocMap(PVBOXVIDEOCM_ALLOC_CONTEXT pContext, PVBOXVIDE
                           NormalPagePriority);
                 if (pvUm)
                 {
+                    pAllocRef->pvUm = pvUm;
                     pAllocRef->pContext = pContext;
                     pAllocRef->pAlloc = pAlloc;
                     pAllocRef->fUhgsmiType = pUmAlloc->fUhgsmiType;
@@ -1196,6 +1219,8 @@ NTSTATUS vboxVideoAMgrCtxAllocMap(PVBOXVIDEOCM_ALLOC_CONTEXT pContext, PVBOXVIDE
                         pUmAlloc->pvData = (uint64_t)pvUm;
                         return STATUS_SUCCESS;
                     }
+
+                    MmUnmapLockedPages(pvUm, &pAllocRef->Mdl);
                 }
                 else
                 {
@@ -1237,6 +1262,8 @@ NTSTATUS vboxVideoAMgrCtxAllocUnmap(PVBOXVIDEOCM_ALLOC_CONTEXT pContext, VBOXDIS
         /* wait for the dereference, i.e. for all commands involving this allocation to complete */
         vboxWddmCounterU32Wait(&pAllocRef->cRefs, 1);
 
+        MmUnmapLockedPages(pAllocRef->pvUm, &pAllocRef->Mdl);
+
         MmUnlockPages(&pAllocRef->Mdl);
         *ppAlloc = pAllocRef->pAlloc;
         if (pAllocRef->pSynchEvent)
@@ -1256,7 +1283,8 @@ static PVBOXVIDEOCM_ALLOC_REF vboxVideoAMgrCtxAllocRefAcquire(PVBOXVIDEOCM_ALLOC
 {
     ExAcquireFastMutex(&pContext->Mutex);
     PVBOXVIDEOCM_ALLOC_REF pAllocRef = (PVBOXVIDEOCM_ALLOC_REF)vboxWddmHTableGet(&pContext->AllocTable, hSesionHandle);
-    ASMAtomicIncU32(&pAllocRef->cRefs);
+    if (pAllocRef)
+        ASMAtomicIncU32(&pAllocRef->cRefs);
     ExReleaseFastMutex(&pContext->Mutex);
     return pAllocRef;
 }
@@ -1919,10 +1947,9 @@ NTSTATUS VBoxWddmSlGetScanLine(PVBOXMP_DEVEXT pDevExt, DXGKARG_GETSCANLINE *pGet
 {
     Assert((UINT)VBoxCommonFromDeviceExt(pDevExt)->cDisplays > pGetScanLine->VidPnTargetId);
     VBOXWDDM_TARGET *pTarget = &pDevExt->aTargets[pGetScanLine->VidPnTargetId];
-    Assert(pTarget->HeightTotal);
-    Assert(pTarget->HeightVisible);
-    Assert(pTarget->HeightTotal >= pTarget->HeightVisible);
-    if (pTarget->HeightTotal)
+    Assert(pTarget->Size.cx);
+    Assert(pTarget->Size.cy);
+    if (pTarget->Size.cy)
     {
         uint32_t curScanLine;
         BOOL bVBlank;
@@ -1940,19 +1967,19 @@ NTSTATUS VBoxWddmSlGetScanLine(PVBOXMP_DEVEXT pDevExt, DXGKARG_GETSCANLINE *pGet
         {
             VSyncTime.QuadPart = VSyncTime.QuadPart - DevVSyncTime.QuadPart;
             /* time is in 100ns, */
-            curScanLine = (uint32_t)((pTarget->HeightTotal * VSyncTime.QuadPart) / DevVSyncTime.QuadPart);
+            curScanLine = (uint32_t)((pTarget->Size.cy * VSyncTime.QuadPart) / DevVSyncTime.QuadPart);
             if (pDevExt->bVSyncTimerEnabled)
             {
-                if (curScanLine >= pTarget->HeightTotal)
+                if (curScanLine >= pTarget->Size.cy)
                     curScanLine = 0;
             }
             else
             {
-                curScanLine %= pTarget->HeightTotal;
+                curScanLine %= pTarget->Size.cy;
             }
         }
 
-        bVBlank = (!curScanLine || curScanLine > pTarget->HeightVisible);
+        bVBlank = (!curScanLine || curScanLine > pTarget->Size.cy);
         pGetScanLine->ScanLine = curScanLine;
         pGetScanLine->InVerticalBlank = bVBlank;
     }
@@ -2065,25 +2092,16 @@ void vboxWddmDiToAllocData(PVBOXMP_DEVEXT pDevExt, const DXGK_DISPLAY_INFORMATIO
             vboxWddmVramAddrToOffset(pDevExt, pInfo->PhysicAddress));
 }
 
-void vboxWddmDmAdjustDefaultVramLocations(PVBOXMP_DEVEXT pDevExt, D3DDDI_VIDEO_PRESENT_SOURCE_ID ModifiedVidPnSourceId)
+void vboxWddmDmSetupDefaultVramLocation(PVBOXMP_DEVEXT pDevExt, D3DDDI_VIDEO_PRESENT_SOURCE_ID ModifiedVidPnSourceId, VBOXWDDM_SOURCE *paSources)
 {
-    PVBOXWDDM_SOURCE pSource = &pDevExt->aSources[ModifiedVidPnSourceId];
-    PHYSICAL_ADDRESS PhAddr;
-    AssertRelease(pSource->AllocData.Addr.SegmentId);
-    AssertRelease(pSource->AllocData.Addr.offVram != VBOXVIDEOOFFSET_VOID);
-    PhAddr.QuadPart = pSource->AllocData.Addr.offVram;
+    PVBOXWDDM_SOURCE pSource = &paSources[ModifiedVidPnSourceId];
+    AssertRelease(g_VBoxDisplayOnly);
+    ULONG offVram = vboxWddmVramCpuVisibleSegmentSize(pDevExt);
+    offVram /= VBoxCommonFromDeviceExt(pDevExt)->cDisplays;
+    offVram &= ~PAGE_OFFSET_MASK;
+    offVram *= ModifiedVidPnSourceId;
 
-    for (UINT i = ModifiedVidPnSourceId + 1; i < (UINT)VBoxCommonFromDeviceExt(pDevExt)->cDisplays; ++i)
-    {
-        /* increaze the phaddr based on the previous source size info */
-        PhAddr.QuadPart += pSource->AllocData.SurfDesc.cbSize;
-        PhAddr.QuadPart = ROUND_TO_PAGES(PhAddr.QuadPart);
-        pSource = &pDevExt->aSources[i];
-        if (pSource->AllocData.Addr.offVram != PhAddr.QuadPart
-                || pSource->AllocData.Addr.SegmentId != 1)
-            pSource->u8SyncState &= ~VBOXWDDM_HGSYNC_F_SYNCED_LOCATION;
-        pSource->AllocData.Addr.SegmentId = 1;
-        pSource->AllocData.Addr.offVram = PhAddr.QuadPart;
-    }
+    if (vboxWddmAddrSetVram(&pSource->AllocData.Addr, 1, offVram))
+        pSource->u8SyncState &= ~VBOXWDDM_HGSYNC_F_SYNCED_LOCATION;
 }
 #endif
