@@ -1,9 +1,10 @@
+/* $Id: DevVGA_VBVA.cpp $ */
 /** @file
  * VirtualBox Video Acceleration (VBVA).
  */
 
 /*
- * Copyright (C) 2006-2012 Oracle Corporation
+ * Copyright (C) 2006-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -14,6 +15,9 @@
  * hope that it will be useful, but WITHOUT ANY WARRANTY of any kind.
  */
 
+/*******************************************************************************
+*   Header Files                                                               *
+*******************************************************************************/
 #define LOG_GROUP LOG_GROUP_DEV_VGA
 #include <VBox/vmm/pdmifs.h>
 #include <VBox/vmm/pdmdev.h>
@@ -36,9 +40,12 @@
 #if 0 // def DEBUG_sunlover
 #define LOGVBVABUFFER(a) LogFlow(a)
 #else
-#define LOGVBVABUFFER(a) do {} while(0)
+#define LOGVBVABUFFER(a) do {} while (0)
 #endif
 
+/*******************************************************************************
+*   Structures and Typedefs                                                    *
+*******************************************************************************/
 typedef struct VBVAPARTIALRECORD
 {
     uint8_t *pu8;
@@ -68,20 +75,23 @@ typedef struct VBVAMOUSESHAPEINFO
     uint8_t *pu8Shape;
 } VBVAMOUSESHAPEINFO;
 
-/* @todo saved state: save and restore VBVACONTEXT */
+/** @todo saved state: save and restore VBVACONTEXT */
 typedef struct VBVACONTEXT
 {
     uint32_t cViews;
     VBVAVIEW aViews[64 /* @todo SchemaDefs::MaxGuestMonitors*/];
     VBVAMOUSESHAPEINFO mouseShapeInfo;
+    bool fPaused;
 } VBVACONTEXT;
 
-/* Copies 'cb' bytes from the VBVA ring buffer to the 'pu8Dst'.
+
+
+/** Copies @a cb bytes from the VBVA ring buffer to the @a pu8Dst.
  * Used for partial records or for records which cross the ring boundary.
  */
 static void vbvaFetchBytes (VBVABUFFER *pVBVA, uint8_t *pu8Dst, uint32_t cb)
 {
-    /* @todo replace the 'if' with an assert. The caller must ensure this condition. */
+    /** @todo replace the 'if' with an assert. The caller must ensure this condition. */
     if (cb >= pVBVA->cbData)
     {
         AssertMsgFailed (("cb = 0x%08X, ring buffer size 0x%08X", cb, pVBVA->cbData));
@@ -485,7 +495,7 @@ static int vbvaEnable (unsigned uScreenId, PVGASTATE pVGAState, VBVACONTEXT *pCt
         pVBVA->hostFlags.u32HostEvents = 0;
         pVBVA->hostFlags.u32SupportedOrders = 0;
 
-        rc = pVGAState->pDrv->pfnVBVAEnable (pVGAState->pDrv, uScreenId, &pVBVA->hostFlags);
+        rc = pVGAState->pDrv->pfnVBVAEnable (pVGAState->pDrv, uScreenId, &pVBVA->hostFlags, false);
     }
     else
     {
@@ -512,6 +522,9 @@ static int vbvaEnable (unsigned uScreenId, PVGASTATE pVGAState, VBVACONTEXT *pCt
 
         pCtx->aViews[uScreenId].pVBVA = pVBVA;
         pCtx->aViews[uScreenId].u32VBVAOffset = u32Offset;
+
+        /* VBVA is working so disable the pause. */
+        pCtx->fPaused = false;
     }
 
     return rc;
@@ -791,7 +804,7 @@ static void vbvaVHWACommandComplete(PVGASTATE pVGAState, PVBOXVHWACMD pCommand,
     if (fAsyncCommand)
     {
         Assert(pCommand->Flags & VBOXVHWACMD_FLAG_HG_ASYNCH);
-        vbvaVHWACommandCompleteAsynch(&pVGAState->IVBVACallbacks, pCommand);
+        vbvaVHWACommandCompleteAsync(&pVGAState->IVBVACallbacks, pCommand);
     }
     else
     {
@@ -808,7 +821,7 @@ static void vbvaVHWACommandCompleteAllPending(PVGASTATE pVGAState, int rc)
 
     VBOX_VHWA_PENDINGCMD *pIter, *pNext;
 
-    PDMCritSectEnter(&pVGAState->lock, VERR_SEM_BUSY);
+    PDMCritSectEnter(&pVGAState->CritSect, VERR_SEM_BUSY);
 
     RTListForEachSafe(&pVGAState->pendingVhwaCommands.PendingList, pIter, pNext, VBOX_VHWA_PENDINGCMD, Node)
     {
@@ -821,7 +834,7 @@ static void vbvaVHWACommandCompleteAllPending(PVGASTATE pVGAState, int rc)
         RTMemFree(pIter);
     }
 
-    PDMCritSectLeave(&pVGAState->lock);
+    PDMCritSectLeave(&pVGAState->CritSect);
 }
 
 static void vbvaVHWACommandClearAllPending(PVGASTATE pVGAState)
@@ -831,7 +844,7 @@ static void vbvaVHWACommandClearAllPending(PVGASTATE pVGAState)
 
     VBOX_VHWA_PENDINGCMD *pIter, *pNext;
 
-    PDMCritSectEnter(&pVGAState->lock, VERR_SEM_BUSY);
+    PDMCritSectEnter(&pVGAState->CritSect, VERR_SEM_BUSY);
 
     RTListForEachSafe(&pVGAState->pendingVhwaCommands.PendingList, pIter, pNext, VBOX_VHWA_PENDINGCMD, Node)
     {
@@ -840,7 +853,7 @@ static void vbvaVHWACommandClearAllPending(PVGASTATE pVGAState)
         RTMemFree(pIter);
     }
 
-    PDMCritSectLeave(&pVGAState->lock);
+    PDMCritSectLeave(&pVGAState->CritSect);
 }
 
 static void vbvaVHWACommandPend(PVGASTATE pVGAState, PVBOXVHWACMD pCommand)
@@ -854,15 +867,15 @@ static void vbvaVHWACommandPend(PVGASTATE pVGAState, PVBOXVHWACMD pCommand)
         {
             pCommand->Flags |= VBOXVHWACMD_FLAG_HG_ASYNCH;
             pPend->pCommand = pCommand;
-            PDMCritSectEnter(&pVGAState->lock, VERR_SEM_BUSY);
+            PDMCritSectEnter(&pVGAState->CritSect, VERR_SEM_BUSY);
             if (ASMAtomicUoReadU32(&pVGAState->pendingVhwaCommands.cPending) < VBOX_VHWA_MAX_PENDING_COMMANDS)
             {
                 RTListAppend(&pVGAState->pendingVhwaCommands.PendingList, &pPend->Node);
                 ASMAtomicIncU32(&pVGAState->pendingVhwaCommands.cPending);
-                PDMCritSectLeave(&pVGAState->lock);
+                PDMCritSectLeave(&pVGAState->CritSect);
                 return;
             }
-            PDMCritSectLeave(&pVGAState->lock);
+            PDMCritSectLeave(&pVGAState->CritSect);
             LogRel(("Pending command count has reached its threshold.. completing them all.."));
             RTMemFree(pPend);
         }
@@ -983,13 +996,13 @@ static bool vbvaVHWACheckPendingCommands(PVGASTATE pVGAState)
 
     VBOX_VHWA_PENDINGCMD *pIter, *pNext;
 
-    PDMCritSectEnter(&pVGAState->lock, VERR_SEM_BUSY);
+    PDMCritSectEnter(&pVGAState->CritSect, VERR_SEM_BUSY);
 
     RTListForEachSafe(&pVGAState->pendingVhwaCommands.PendingList, pIter, pNext, VBOX_VHWA_PENDINGCMD, Node)
     {
         if (!vbvaVHWACommandSubmit(pVGAState, pIter->pCommand, true))
         {
-            PDMCritSectLeave(&pVGAState->lock);
+            PDMCritSectLeave(&pVGAState->CritSect);
             return false; /* the command should be pended still */
         }
 
@@ -999,7 +1012,7 @@ static bool vbvaVHWACheckPendingCommands(PVGASTATE pVGAState)
         RTMemFree(pIter);
     }
 
-    PDMCritSectLeave(&pVGAState->lock);
+    PDMCritSectLeave(&pVGAState->CritSect);
 
     return true;
 }
@@ -1230,7 +1243,7 @@ int vboxVBVASaveStateDone (PPDMDEVINS pDevIns, PSSMHANDLE pSSM)
     return vbvaVHWAEnable(PDMINS_2_DATA(pDevIns, PVGASTATE), true);
 }
 
-int vbvaVHWACommandCompleteAsynch(PPDMIDISPLAYVBVACALLBACKS pInterface, PVBOXVHWACMD pCmd)
+int vbvaVHWACommandCompleteAsync(PPDMIDISPLAYVBVACALLBACKS pInterface, PVBOXVHWACMD pCmd)
 {
     int rc;
     Log(("VGA Command <<< Async rc %d %#p, %d\n", pCmd->rc, pCmd, pCmd->enmCmd));
@@ -1467,8 +1480,10 @@ static DECLCALLBACK(bool) vboxVBVALoadStatePerformPreCb(PVGASTATE pVGAState, VBO
     switch (u32)
     {
         case VBOXVBVASAVEDSTATE_VHWAAVAILABLE_MAGIC:
+            pData->ab2DOn[iDisplay] = true;
             return true;
         case VBOXVBVASAVEDSTATE_VHWAUNAVAILABLE_MAGIC:
+            pData->ab2DOn[iDisplay] = false;
             return false;
         default:
             pData->rc = VERR_INVALID_STATE;
@@ -1644,6 +1659,9 @@ int vboxVBVASaveStateExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSM)
             AssertRCReturn(rc, rc);
         }
     }
+
+    /* no pending commands */
+    SSMR3PutU32(pSSM, 0);
 #endif
     return rc;
 }
@@ -1805,6 +1823,19 @@ int vboxVBVALoadStateExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSM, uint32_t u32Vers
 
             if (u32Version > VGA_SAVEDSTATE_VERSION_WDDM)
             {
+#define VBOX_VHWA_SOLARIS_ARCH "solaris."
+
+                bool fLoadCommands;
+
+                if (u32Version < VGA_SAVEDSTATE_VERSION_FIXED_PENDVHWA)
+                {
+                    const char *pcszOsArch = SSMR3HandleHostOSAndArch(pSSM);
+                    Assert(pcszOsArch);
+                    fLoadCommands = !pcszOsArch || RTStrNCmp(pcszOsArch, VBOX_VHWA_SOLARIS_ARCH, sizeof (VBOX_VHWA_SOLARIS_ARCH) - 1);
+                }
+                else
+                    fLoadCommands = true;
+
 #ifdef VBOX_WITH_VIDEOHWACCEL
                 uint32_t cbCmd = sizeof (VBOXVHWACMD_HH_SAVESTATE_LOADPERFORM); /* maximum cmd size */
                 VBOXVHWACMD *pCmd = vbvaVHWAHHCommandCreate(pVGAState, VBOXVHWACMD_TYPE_HH_SAVESTATE_LOADPERFORM, 0, cbCmd);
@@ -1820,16 +1851,42 @@ int vboxVBVALoadStateExec (PPDMDEVINS pDevIns, PSSMHANDLE pSSM, uint32_t u32Vers
                     vbvaVHWAHHCommandRelease(pCmd);
                     AssertRCReturn(rc, rc);
 
-                    rc = vbvaVHWACommandLoadPending(pVGAState, pSSM, u32Version);
-                    AssertRCReturn(rc, rc);
+                    if (fLoadCommands)
+                    {
+                        rc = vbvaVHWACommandLoadPending(pVGAState, pSSM, u32Version);
+                        AssertRCReturn(rc, rc);
+                    }
                 }
                 else
                 {
                     rc = VERR_OUT_OF_RESOURCES;
                 }
 #else
-                rc = SSMR3SkipToEndOfUnit(pSSM);
-                AssertRCReturn(rc, rc);
+                uint32_t u32;
+
+                for (uint32_t i = 0; i < pVGAState->cMonitors; ++i)
+                {
+                    rc = SSMR3GetU32(pSSM, &u32);
+                    AssertRCReturn(rc, rc);
+
+                    if (u32 != VBOXVBVASAVEDSTATE_VHWAUNAVAILABLE_MAGIC)
+                    {
+                        LogRel(("2D data while 2D is not supported\n"));
+                        return VERR_NOT_SUPPORTED;
+                    }
+                }
+
+                if (fLoadCommands)
+                {
+                    rc = SSMR3GetU32(pSSM, &u32);
+                    AssertRCReturn(rc, rc);
+
+                    if (u32)
+                    {
+                        LogRel(("2D pending command while 2D is not supported\n"));
+                        return VERR_NOT_SUPPORTED;
+                    }
+                }
 #endif
             }
 
@@ -1856,6 +1913,9 @@ int vboxVBVALoadStateDone (PPDMDEVINS pDevIns, PSSMHANDLE pSSM)
 
             if (pView->pVBVA)
             {
+#ifdef VBOX_WITH_CRHGSMI
+                Assert(!vboxCmdVBVAIsEnabled(pVGAState));
+#endif
                 vbvaEnable (iView, pVGAState, pCtx, pView->pVBVA, pView->u32VBVAOffset, true /* fRestored */);
                 vbvaResize (pVGAState, pView, &pView->screen);
             }
@@ -1873,23 +1933,104 @@ int vboxVBVALoadStateDone (PPDMDEVINS pDevIns, PSSMHANDLE pSSM)
 void VBVARaiseIrq (PVGASTATE pVGAState, uint32_t fFlags)
 {
     PPDMDEVINS pDevIns = pVGAState->pDevInsR3;
-    PDMCritSectEnter(&pVGAState->lock, VERR_SEM_BUSY);
+    PDMCritSectEnter(&pVGAState->CritSect, VERR_SEM_BUSY);
+
     HGSMISetHostGuestFlags(pVGAState->pHGSMI, HGSMIHOSTFLAGS_IRQ | fFlags);
     PDMDevHlpPCISetIrq(pDevIns, 0, PDM_IRQ_LEVEL_HIGH);
-    PDMCritSectLeave(&pVGAState->lock);
+
+    PDMCritSectLeave(&pVGAState->CritSect);
+}
+
+static DECLCALLBACK(int) vbvaRaiseIrqEMT(PVGASTATE pVGAState, uint32_t fFlags)
+{
+    VBVARaiseIrq(pVGAState, fFlags);
+    return VINF_SUCCESS;
 }
 
 void VBVARaiseIrqNoWait(PVGASTATE pVGAState, uint32_t fFlags)
 {
-    PPDMDEVINS pDevIns = pVGAState->pDevInsR3;
-    PDMCritSectEnter(&pVGAState->lock, VERR_SEM_BUSY);
-
-    HGSMISetHostGuestFlags(pVGAState->pHGSMI, HGSMIHOSTFLAGS_IRQ | fFlags);
-    PDMDevHlpPCISetIrqNoWait(pDevIns, 0, PDM_IRQ_LEVEL_HIGH);
-
-    PDMCritSectLeave(&pVGAState->lock);
+    /* we can not use PDMDevHlpPCISetIrqNoWait here, because we need to set IRG host flag and raise IRQ atomically,
+     * otherwise there might be a situation, when:
+     * 1. Flag is set
+     * 2. guest issues an IRQ clean request, that cleans up the flag and the interrupt
+     * 3. IRQ is set */
+    VMR3ReqCallNoWait(PDMDevHlpGetVM(pVGAState->pDevInsR3), VMCPUID_ANY, (PFNRT)vbvaRaiseIrqEMT, 2, pVGAState, fFlags);
 }
 
+int VBVAInfoView(PVGASTATE pVGAState, VBVAINFOVIEW *pView)
+{
+    LogFlowFunc(("VBVA_INFO_VIEW: index %d, offset 0x%x, size 0x%x, max screen size 0x%x\n",
+                     pView->u32ViewIndex, pView->u32ViewOffset, pView->u32ViewSize, pView->u32MaxScreenSize));
+
+    PHGSMIINSTANCE pIns = pVGAState->pHGSMI;
+    VBVACONTEXT *pCtx = (VBVACONTEXT *)HGSMIContext (pIns);
+
+    /* @todo verify view data. */
+    if (pView->u32ViewIndex >= pCtx->cViews)
+    {
+        Log(("View index too large %d!!!\n",
+             pView->u32ViewIndex));
+        return VERR_INVALID_PARAMETER;
+    }
+
+    pCtx->aViews[pView->u32ViewIndex].view = *pView;
+
+    return VINF_SUCCESS;
+}
+
+int VBVAInfoScreen(PVGASTATE pVGAState, VBVAINFOSCREEN *pScreen)
+{
+    PHGSMIINSTANCE pIns = pVGAState->pHGSMI;
+    VBVACONTEXT *pCtx = (VBVACONTEXT *)HGSMIContext (pIns);
+    VBVAINFOVIEW *pView = &pCtx->aViews[pScreen->u32ViewIndex].view;
+    /* Calculate the offset of the  end of the screen so we can make
+     * sure it is inside the view.  I assume that screen rollover is not
+     * implemented. */
+    int64_t offEnd =   (int64_t)pScreen->u32Height * pScreen->u32LineSize
+                     + pScreen->u32Width + pScreen->u32StartOffset;
+    LogRel(("VBVA_INFO_SCREEN: [%d] @%d,%d %dx%d, line 0x%x, BPP %d, flags 0x%x\n",
+                    pScreen->u32ViewIndex, pScreen->i32OriginX, pScreen->i32OriginY,
+                    pScreen->u32Width, pScreen->u32Height,
+                    pScreen->u32LineSize,  pScreen->u16BitsPerPixel, pScreen->u16Flags));
+
+    if (   pScreen->u32ViewIndex < RT_ELEMENTS (pCtx->aViews)
+        && pScreen->u16BitsPerPixel <= 32
+        && pScreen->u32Width <= UINT16_MAX
+        && pScreen->u32Height <= UINT16_MAX
+        && pScreen->u32LineSize <= UINT16_MAX * 4
+        && offEnd < pView->u32MaxScreenSize)
+    {
+        vbvaResize (pVGAState, &pCtx->aViews[pScreen->u32ViewIndex], pScreen);
+        return VINF_SUCCESS;
+    }
+
+    LogRelFlow(("VBVA_INFO_SCREEN [%lu]: bad data: %lux%lu, line 0x%lx, BPP %u, start offset %lu, max screen size %lu\n",
+                    (unsigned long)pScreen->u32ViewIndex,
+                    (unsigned long)pScreen->u32Width,
+                    (unsigned long)pScreen->u32Height,
+                    (unsigned long)pScreen->u32LineSize,
+                    (unsigned long)pScreen->u16BitsPerPixel,
+                    (unsigned long)pScreen->u32StartOffset,
+                    (unsigned long)pView->u32MaxScreenSize));
+    return VERR_INVALID_PARAMETER;
+}
+
+int VBVAGetInfoViewAndScreen(PVGASTATE pVGAState, uint32_t u32ViewIndex, VBVAINFOVIEW *pView, VBVAINFOSCREEN *pScreen)
+{
+    if (u32ViewIndex >= pVGAState->cMonitors)
+        return VERR_INVALID_PARAMETER;
+
+    PHGSMIINSTANCE pIns = pVGAState->pHGSMI;
+    VBVACONTEXT *pCtx = (VBVACONTEXT *)HGSMIContext (pIns);
+
+    if (pView)
+        *pView = pCtx->aViews[u32ViewIndex].view;
+
+    if (pScreen)
+        *pScreen = pCtx->aViews[u32ViewIndex].screen;
+
+    return VINF_SUCCESS;
+}
 
 /*
  *
@@ -1927,6 +2068,37 @@ static DECLCALLBACK(int) vbvaChannelHandler (void *pvHandler, uint16_t u16Channe
 
     switch (u16ChannelInfo)
     {
+        case VBVA_CMDVBVA_SUBMIT:
+        {
+# ifdef VBOX_WITH_CRHGSMI
+            rc = vboxCmdVBVACmdSubmit(pVGAState);
+#endif
+            break;
+        }
+        case VBVA_CMDVBVA_FLUSH:
+        {
+# ifdef VBOX_WITH_CRHGSMI
+            rc =vboxCmdVBVACmdFlush(pVGAState);
+#endif
+            break;
+        }
+        case VBVA_CMDVBVA_CTL:
+        {
+#ifdef VBOX_WITH_CRHGSMI
+            if (cbBuffer < VBoxSHGSMIBufferHeaderSize() + sizeof (VBOXCMDVBVA_CTL))
+            {
+                Log(("buffer too small\n"));
+#ifdef DEBUG_misha
+                AssertMsgFailed(("buffer too small\n"));
+#endif
+                rc = VERR_INVALID_PARAMETER;
+                break;
+            }
+            VBOXCMDVBVA_CTL *pCtl = (VBOXCMDVBVA_CTL*)VBoxSHGSMIBufferData((PVBOXSHGSMIHEADER)pvBuffer);
+            rc = vboxCmdVBVACmdCtl(pVGAState, pCtl, cbBuffer - VBoxSHGSMIBufferHeaderSize());
+#endif
+            break;
+        }
 #ifdef VBOX_WITH_VDMA
         case VBVA_VDMA_CMD:
         {
@@ -2012,6 +2184,15 @@ static DECLCALLBACK(int) vbvaChannelHandler (void *pvHandler, uint16_t u16Channe
 
         case VBVA_INFO_VIEW:
         {
+#ifdef VBOX_WITH_CRHGSMI
+            if (vboxCmdVBVAIsEnabled(pVGAState))
+            {
+                AssertMsgFailed(("VBVA_INFO_VIEW is not acceptible for CmdVbva\n"));
+                rc = VERR_INVALID_PARAMETER;
+                break;
+            }
+#endif
+
             if (cbBuffer < sizeof (VBVAINFOVIEW))
             {
                 rc = VERR_INVALID_PARAMETER;
@@ -2025,19 +2206,10 @@ static DECLCALLBACK(int) vbvaChannelHandler (void *pvHandler, uint16_t u16Channe
                  cbBuffer >= sizeof (VBVAINFOVIEW);
                  pView++, cbBuffer -= sizeof (VBVAINFOVIEW))
             {
-                LogFlowFunc(("VBVA_INFO_VIEW: index %d, offset 0x%x, size 0x%x, max screen size 0x%x\n",
-                             pView->u32ViewIndex, pView->u32ViewOffset, pView->u32ViewSize, pView->u32MaxScreenSize));
-
-                /* @todo verify view data. */
-                if (pView->u32ViewIndex >= pCtx->cViews)
-                {
-                    Log(("View index too large %d!!!\n",
-                         pView->u32ViewIndex));
-                    rc = VERR_INVALID_PARAMETER;
+                VBVAINFOVIEW View = *pView;
+                rc = VBVAInfoView(pVGAState, &View);
+                if (RT_FAILURE(rc))
                     break;
-                }
-
-                pCtx->aViews[pView->u32ViewIndex].view = *pView;
             }
         } break;
 
@@ -2079,43 +2251,31 @@ static DECLCALLBACK(int) vbvaChannelHandler (void *pvHandler, uint16_t u16Channe
                 break;
             }
 
-            VBVAINFOSCREEN *pScreen = (VBVAINFOSCREEN *)pvBuffer;
-            VBVAINFOVIEW *pView = &pCtx->aViews[pScreen->u32ViewIndex].view;
-            /* Calculate the offset of the  end of the screen so we can make
-             * sure it is inside the view.  I assume that screen rollover is not
-             * implemented. */
-            int64_t offEnd =   (int64_t)pScreen->u32Height * pScreen->u32LineSize
-                             + pScreen->u32Width + pScreen->u32StartOffset;
-            LogRel(("VBVA_INFO_SCREEN: [%d] @%d,%d %dx%d, line 0x%x, BPP %d, flags 0x%x\n",
-                            pScreen->u32ViewIndex, pScreen->i32OriginX, pScreen->i32OriginY,
-                            pScreen->u32Width, pScreen->u32Height,
-                            pScreen->u32LineSize,  pScreen->u16BitsPerPixel, pScreen->u16Flags));
-
-            if (   pScreen->u32ViewIndex < RT_ELEMENTS (pCtx->aViews)
-                && pScreen->u16BitsPerPixel <= 32
-                && pScreen->u32Width <= UINT16_MAX
-                && pScreen->u32Height <= UINT16_MAX
-                && pScreen->u32LineSize <= UINT16_MAX * 4
-                && offEnd < pView->u32MaxScreenSize)
+#ifdef VBOX_WITH_CRHGSMI
+            if (vboxCmdVBVAIsEnabled(pVGAState))
             {
-                vbvaResize (pVGAState, &pCtx->aViews[pScreen->u32ViewIndex], pScreen);
-            }
-            else
-            {
-                LogRelFlow(("VBVA_INFO_SCREEN [%lu]: bad data: %lux%lu, line 0x%lx, BPP %u, start offset %lu, max screen size %lu\n",
-                            (unsigned long)pScreen->u32ViewIndex,
-                            (unsigned long)pScreen->u32Width,
-                            (unsigned long)pScreen->u32Height,
-                            (unsigned long)pScreen->u32LineSize,
-                            (unsigned long)pScreen->u16BitsPerPixel,
-                            (unsigned long)pScreen->u32StartOffset,
-                            (unsigned long)pView->u32MaxScreenSize));
+                AssertMsgFailed(("VBVA_INFO_SCREEN is not acceptible for CmdVbva\n"));
                 rc = VERR_INVALID_PARAMETER;
+                break;
             }
+#endif
+
+            VBVAINFOSCREEN *pScreen = (VBVAINFOSCREEN *)pvBuffer;
+            VBVAINFOSCREEN Screen = *pScreen;
+            rc = VBVAInfoScreen(pVGAState, &Screen);
         } break;
 
         case VBVA_ENABLE:
         {
+#ifdef VBOX_WITH_CRHGSMI
+            if (vboxCmdVBVAIsEnabled(pVGAState))
+            {
+                AssertMsgFailed(("VBVA_ENABLE is not acceptible for CmdVbva\n"));
+                rc = VERR_INVALID_PARAMETER;
+                break;
+            }
+#endif
+
             if (cbBuffer < sizeof (VBVAENABLE))
             {
                 rc = VERR_INVALID_PARAMETER;
@@ -2240,54 +2400,6 @@ static DECLCALLBACK(int) vbvaChannelHandler (void *pvHandler, uint16_t u16Channe
             pCaps->rc = VINF_SUCCESS;
         } break;
 #endif
-
-        case VBVA_CMDVBVA_ENABLE:
-        {
-            if (cbBuffer < sizeof (VBVAENABLE))
-            {
-                rc = VERR_INVALID_PARAMETER;
-                break;
-            }
-
-            VBVAENABLE *pEnable = (VBVAENABLE *)pvBuffer;
-
-            if ((pEnable->u32Flags & (VBVA_F_ENABLE | VBVA_F_DISABLE)) == VBVA_F_ENABLE)
-            {
-                uint32_t u32Offset = pEnable->u32Offset;
-                VBVABUFFER *pVBVA = (VBVABUFFER *)HGSMIOffsetToPointerHost (pIns, u32Offset);
-
-                if (pVBVA)
-                    rc = vboxCmdVBVAEnable(pVGAState, pVBVA);
-                else
-                {
-                    LogRel(("Invalid VBVABUFFER offset 0x%x!!!\n",
-                         pEnable->u32Offset));
-                    rc = VERR_INVALID_PARAMETER;
-                }
-            }
-            else if ((pEnable->u32Flags & (VBVA_F_ENABLE | VBVA_F_DISABLE)) == VBVA_F_DISABLE)
-            {
-                rc = vboxCmdVBVADisable(pVGAState);
-            }
-            else
-            {
-                LogRel(("Invalid VBVA_ENABLE flags 0x%x!!!\n", pEnable->u32Flags));
-                rc = VERR_INVALID_PARAMETER;
-            }
-
-            pEnable->i32Result = rc;
-            break;
-        }
-        case VBVA_CMDVBVA_SUBMIT:
-        {
-            rc = vboxCmdVBVACmdSubmit(pVGAState);
-            break;
-        }
-        case VBVA_CMDVBVA_FLUSH:
-        {
-            rc =vboxCmdVBVACmdFlush(pVGAState);
-            break;
-        }
         case VBVA_SCANLINE_CFG:
         {
             if (cbBuffer < sizeof (VBVASCANLINECFG))
@@ -2309,6 +2421,24 @@ static DECLCALLBACK(int) vbvaChannelHandler (void *pvHandler, uint16_t u16Channe
     return rc;
 }
 
+/* When VBVA is paused, then VGA device is allowed to work but
+ * no HGSMI etc state is changed.
+ */
+void VBVAPause(PVGASTATE pVGAState, bool fPause)
+{
+    if (!pVGAState || !pVGAState->pHGSMI)
+    {
+        return;
+    }
+
+    VBVACONTEXT *pCtx = (VBVACONTEXT *)HGSMIContext(pVGAState->pHGSMI);
+
+    if (pCtx)
+    {
+        pCtx->fPaused = fPause;
+    }
+}
+
 void VBVAReset (PVGASTATE pVGAState)
 {
     if (!pVGAState || !pVGAState->pHGSMI)
@@ -2357,14 +2487,17 @@ int VBVAUpdateDisplay (PVGASTATE pVGAState)
 
     if (pCtx)
     {
-        rc = vbvaFlush (pVGAState, pCtx);
-
-        if (RT_SUCCESS (rc))
+        if (!pCtx->fPaused)
         {
-            if (!pCtx->aViews[0].pVBVA)
+            rc = vbvaFlush (pVGAState, pCtx);
+
+            if (RT_SUCCESS (rc))
             {
-                /* VBVA is not enabled for the first view, so VGA device must do updates. */
-                rc = VERR_NOT_SUPPORTED;
+                if (!pCtx->aViews[0].pVBVA)
+                {
+                    /* VBVA is not enabled for the first view, so VGA device must do updates. */
+                    rc = VERR_NOT_SUPPORTED;
+                }
             }
         }
     }
@@ -2401,6 +2534,7 @@ int VBVAInit (PVGASTATE pVGAState)
          {
              VBVACONTEXT *pCtx = (VBVACONTEXT *)HGSMIContext (pVGAState->pHGSMI);
              pCtx->cViews = pVGAState->cMonitors;
+             pCtx->fPaused = true;
          }
      }
 
@@ -2424,39 +2558,3 @@ void VBVADestroy (PVGASTATE pVGAState)
     HGSMIDestroy (pVGAState->pHGSMI);
     pVGAState->pHGSMI = NULL;
 }
-
-int VBVAGetScreenInfo(PVGASTATE pVGAState, unsigned uScreenId, struct VBVAINFOSCREEN *pScreen, void **ppvVram)
-{
-    PPDMDEVINS pDevIns = pVGAState->pDevInsR3;
-    PHGSMIINSTANCE pIns = pVGAState->pHGSMI;
-    VBVACONTEXT *pCtx = (VBVACONTEXT *)HGSMIContext (pIns);
-    int rc = PDMCritSectEnter(&pVGAState->lock, VERR_SEM_BUSY);
-    if (RT_SUCCESS(rc))
-    {
-        if (uScreenId < pCtx->cViews)
-        {
-            VBVAVIEW *pView = &pCtx->aViews[uScreenId];
-            if (pView->pVBVA)
-            {
-                uint8_t *pu8VRAM = pVGAState->vram_ptrR3 + pView->view.u32ViewOffset;
-                *pScreen = pView->screen;
-                *ppvVram = (void*)pu8VRAM;
-                rc = VINF_SUCCESS;
-            }
-            else
-            {
-                /* pretend disabled */
-                memset(pScreen, 0, sizeof (*pScreen));
-                pScreen->u16Flags = VBVA_SCREEN_F_DISABLED;
-                pScreen->u32ViewIndex = uScreenId;
-                *ppvVram = NULL;
-                rc = VINF_SUCCESS;
-            }
-        }
-        else
-            rc = VERR_INVALID_PARAMETER;
-
-        PDMCritSectLeave(&pVGAState->lock);
-    }
-    return rc;
-}
