@@ -4,7 +4,7 @@
  */
 
 /*
- * Copyright (C) 2006-2010 Oracle Corporation
+ * Copyright (C) 2006-2012 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -191,6 +191,10 @@ ip_input(PNATState pData, struct mbuf *m)
     if (mlen > ip->ip_len)
         m_adj(m, ip->ip_len - m->m_len);
 
+    /* source must be unicast */
+    if ((ip->ip_src.s_addr & RT_N2H_U32_C(0xe0000000)) == RT_N2H_U32_C(0xe0000000))
+        goto free_m;
+
     /* check ip_ttl for a correct ICMP reply */
     if (ip->ip_ttl==0 || ip->ip_ttl == 1)
     {
@@ -199,6 +203,18 @@ ip_input(PNATState pData, struct mbuf *m)
     }
 
     ip->ip_ttl--;
+
+    /*
+     * Drop multicast (class d) and reserved (class e) here.  The rest
+     * of the code is not yet prepared to deal with it.  IGMP is not
+     * implemented either.
+     */
+    if (   (ip->ip_dst.s_addr & RT_N2H_U32_C(0xe0000000)) == RT_N2H_U32_C(0xe0000000)
+        && ip->ip_dst.s_addr != 0xffffffff)
+    {
+        goto free_m;
+    }
+
     /*
      * If offset or IP_MF are set, must reassemble.
      * Otherwise, nothing need be done.
@@ -242,6 +258,7 @@ ip_input(PNATState pData, struct mbuf *m)
 bad_free_m:
     Log2(("NAT: IP datagram to %RTnaipv4 with size(%d) claimed as bad\n",
         ip->ip_dst, ip->ip_len));
+free_m:
     m_freem(pData, m);
 no_free_m:
     STAM_PROFILE_STOP(&pData->StatIP_input, a);
