@@ -1,12 +1,10 @@
 /* $Id: UIMachineMenuBar.cpp $ */
 /** @file
- *
- * VBox frontends: Qt GUI ("VirtualBox"):
- * UIMachineMenuBar class implementation
+ * VBox Qt GUI - UIMachineMenuBar class implementation.
  */
 
 /*
- * Copyright (C) 2010-2012 Oracle Corporation
+ * Copyright (C) 2010-2013 Oracle Corporation
  *
  * This file is part of VirtualBox Open Source Edition (OSE), as
  * available from http://www.virtualbox.org. This file is free software;
@@ -29,51 +27,64 @@
 #include "UIActionPoolRuntime.h"
 #include "VBoxGlobal.h"
 #include "UIMessageCenter.h"
-#include "UIExtraDataEventHandler.h"
 #include "UIImageTools.h"
-#include "UINetworkManager.h"
-#include "VBoxGlobal.h"
-#include "UISession.h"
+#ifdef VBOX_GUI_WITH_NETWORK_MANAGER
+# include "UINetworkManager.h"
+#endif /* VBOX_GUI_WITH_NETWORK_MANAGER */
 
 /* COM includes: */
 #include "CMachine.h"
 
-/* Helper QMenu reimplementation which allows
- * to highlight first menu item for popped up menu: */
+
+/**
+ * QMenu sub-class with extended functionality.
+ * Allows to highlight first menu item for popped up menu.
+ */
 class QIMenu : public QMenu
 {
     Q_OBJECT;
 
 public:
 
+    /** Constructor. */
     QIMenu() : QMenu(0) {}
 
 private slots:
 
-    void sltSelectFirstAction()
+    /** Highlights first menu action for popped up menu. */
+    void sltHighlightFirstAction()
     {
 #ifdef Q_WS_WIN
         activateWindow();
-#endif
+#endif /* Q_WS_WIN */
         QMenu::focusNextChild();
     }
 };
 
+
+/**
+ * QMenuBar sub-class with extended functionality.
+ * Reflects BETA label when necessary.
+ */
 class UIMenuBar: public QMenuBar
 {
+    Q_OBJECT;
+
 public:
 
+    /** Constructor. */
     UIMenuBar(QWidget *pParent = 0)
-      : QMenuBar(pParent)
-      , m_fShowBetaLabel(false)
+        : QMenuBar(pParent)
+        , m_fShowBetaLabel(false)
     {
-        /* Check for beta versions */
+        /* Check for beta versions: */
         if (vboxGlobal().isBeta())
             m_fShowBetaLabel = true;
     }
 
 protected:
 
+    /** Paint-event reimplementation. */
     void paintEvent(QPaintEvent *pEvent)
     {
         QMenuBar::paintEvent(pEvent);
@@ -95,16 +106,13 @@ protected:
 
 private:
 
-    /* Private member vars */
+    /** Reflects whether we should show BETA label or not. */
     bool m_fShowBetaLabel;
 };
 
-UIMachineMenuBar::UIMachineMenuBar(UISession *pSession, const CMachine &machine)
-    /* On the Mac we add some items only the first time, cause otherwise they
-     * will be merged more than once to the application menu by Qt. */
-    : m_fIsFirstTime(true)
-    , m_pSession(pSession)
-    , m_machine(machine)
+
+UIMachineMenuBar::UIMachineMenuBar(UISession *pSession)
+    : m_pSession(pSession)
 {
 }
 
@@ -167,8 +175,7 @@ QList<QMenu*> UIMachineMenuBar::prepareSubMenus(RuntimeMenuType fOptions /* = Ru
     /* Debug submenu: */
     if (fOptions & RuntimeMenuType_Debug)
     {
-        CMachine machine; /** @todo we should try get the machine here. But we'll
-                           *        probably be fine with the cached values. */
+        CMachine machine = m_pSession->session().GetMachine();
         if (vboxGlobal().isDebuggerEnabled(machine))
         {
             QMenu *pMenuDebug = gActionPool->action(UIActionIndexRuntime_Menu_Debug)->menu();
@@ -176,7 +183,7 @@ QList<QMenu*> UIMachineMenuBar::prepareSubMenus(RuntimeMenuType fOptions /* = Ru
             preparedSubMenus << pMenuDebug;
         }
     }
-#endif
+#endif /* VBOX_WITH_DEBUGGER_GUI */
 
     /* Help submenu: */
     if (fOptions & RuntimeMenuType_Help)
@@ -210,7 +217,7 @@ void UIMachineMenuBar::prepareMenuMachine(QMenu *pMenu)
         gActionPool->action(UIActionIndexRuntime_Simple_SettingsDialog)->setEnabled(false);
     /* Take Snapshot action: */
     if (m_pSession->allowedActionsMenuMachine() & RuntimeMenuMachineActionType_TakeSnapshot &&
-        vboxGlobal().shouldWeAllowSnapshotOperations(m_machine, false /* sanity check? */))
+        m_pSession->isSnapshotOperationsAllowed())
     {
         pMenu->addAction(gActionPool->action(UIActionIndexRuntime_Simple_TakeSnapshot));
         fSeparator1 = true;
@@ -349,10 +356,12 @@ void UIMachineMenuBar::prepareMenuView(QMenu *pMenu)
     gActionPool->action(UIActionIndexRuntime_Toggle_Fullscreen)->setEnabled(fIsAllowedFullscreen);
     if (fIsAllowedFullscreen)
         pMenu->addAction(gActionPool->action(UIActionIndexRuntime_Toggle_Fullscreen));
+
     /* Seamless action: */
     gActionPool->action(UIActionIndexRuntime_Toggle_Seamless)->setEnabled(fIsAllowedSeamless);
     if (fIsAllowedSeamless)
         pMenu->addAction(gActionPool->action(UIActionIndexRuntime_Toggle_Seamless));
+
     /* Scale action: */
     gActionPool->action(UIActionIndexRuntime_Toggle_Scale)->setEnabled(fIsAllowedScale);
     if (fIsAllowedScale)
@@ -368,6 +377,7 @@ void UIMachineMenuBar::prepareMenuView(QMenu *pMenu)
         pMenu->addAction(gActionPool->action(UIActionIndexRuntime_Toggle_GuestAutoresize));
     else
         gActionPool->action(UIActionIndexRuntime_Toggle_GuestAutoresize)->setEnabled(false);
+
     /* Adjust Window action: */
     if (m_pSession->allowedActionsMenuView() & RuntimeMenuViewActionType_AdjustWindow)
         pMenu->addAction(gActionPool->action(UIActionIndexRuntime_Simple_AdjustWindow));
@@ -409,6 +419,14 @@ void UIMachineMenuBar::prepareMenuDevices(QMenu *pMenu)
     }
     else
         gActionPool->action(UIActionIndexRuntime_Menu_USBDevices)->setEnabled(false);
+    /* Web Cams submenu: */
+    if (m_pSession->allowedActionsMenuDevices() & RuntimeMenuDevicesActionType_WebCams)
+    {
+        pMenu->addAction(gActionPool->action(UIActionIndexRuntime_Menu_WebCams));
+        fSeparator1 = true;
+    }
+    else
+        gActionPool->action(UIActionIndexRuntime_Menu_WebCams)->setEnabled(false);
     /* Shared Clipboard submenu: */
     if (m_pSession->allowedActionsMenuDevices() & RuntimeMenuDevicesActionType_SharedClipboard)
     {
@@ -425,33 +443,56 @@ void UIMachineMenuBar::prepareMenuDevices(QMenu *pMenu)
     }
     else
         gActionPool->action(UIActionIndexRuntime_Menu_DragAndDrop)->setEnabled(false);
-    /* Network Settings action: */
+    /* Network submenu: */
     if (m_pSession->allowedActionsMenuDevices() & RuntimeMenuDevicesActionType_NetworkSettings)
     {
-        pMenu->addAction(gActionPool->action(UIActionIndexRuntime_Simple_NetworkAdaptersDialog));
+        pMenu->addAction(gActionPool->action(UIActionIndexRuntime_Menu_Network));
+        gActionPool->action(UIActionIndexRuntime_Menu_Network)->menu()->addAction(gActionPool->action(UIActionIndexRuntime_Simple_NetworkSettings));
         fSeparator1 = true;
     }
     else
-        gActionPool->action(UIActionIndexRuntime_Simple_NetworkAdaptersDialog)->setEnabled(false);
+    {
+        gActionPool->action(UIActionIndexRuntime_Menu_Network)->setEnabled(false);
+        gActionPool->action(UIActionIndexRuntime_Simple_NetworkSettings)->setEnabled(false);
+    }
     /* Shared Folders Settings action: */
     if (m_pSession->allowedActionsMenuDevices() & RuntimeMenuDevicesActionType_SharedFoldersSettings)
     {
-        pMenu->addAction(gActionPool->action(UIActionIndexRuntime_Simple_SharedFoldersDialog));
+        pMenu->addAction(gActionPool->action(UIActionIndexRuntime_Simple_SharedFoldersSettings));
         fSeparator1 = true;
     }
     else
-        gActionPool->action(UIActionIndexRuntime_Simple_SharedFoldersDialog)->setEnabled(false);
+        gActionPool->action(UIActionIndexRuntime_Simple_SharedFoldersSettings)->setEnabled(false);
+
+    /* Separator #1: */
+    if (fSeparator1)
+        pMenu->addSeparator();
+
+
+    /* Separator #2? */
+    bool fSeparator2 = false;
+
     /* VRDE Server action: */
     if (m_pSession->allowedActionsMenuDevices() & RuntimeMenuDevicesActionType_VRDEServer)
     {
         pMenu->addAction(gActionPool->action(UIActionIndexRuntime_Toggle_VRDEServer));
-        fSeparator1 = true;
+        if (!m_pSession->isExtensionPackUsable())
+            gActionPool->action(UIActionIndexRuntime_Toggle_VRDEServer)->setEnabled(false);
+        fSeparator2 = true;
     }
     else
         gActionPool->action(UIActionIndexRuntime_Toggle_VRDEServer)->setEnabled(false);
+    /* Video Capture action: */
+    if (m_pSession->allowedActionsMenuDevices() & RuntimeMenuDevicesActionType_VideoCapture)
+    {
+        pMenu->addAction(gActionPool->action(UIActionIndexRuntime_Toggle_VideoCapture));
+        fSeparator2 = true;
+    }
+    else
+        gActionPool->action(UIActionIndexRuntime_Toggle_VideoCapture)->setEnabled(false);
 
-    /* Separator #1: */
-    if (fSeparator1)
+    /* Separator #2: */
+    if (fSeparator2)
         pMenu->addSeparator();
 
 
@@ -507,7 +548,7 @@ void UIMachineMenuBar::prepareMenuHelp(QMenu *pMenu)
     {
         pMenu->addAction(gActionPool->action(UIActionIndex_Simple_Contents));
         VBoxGlobal::connect(gActionPool->action(UIActionIndex_Simple_Contents), SIGNAL(triggered()),
-                            &msgCenter(), SLOT(sltShowHelpHelpDialog()), Qt::UniqueConnection);
+                            &msgCenter(), SLOT(sltShowHelpHelpDialog()));
         fSeparator1 = true;
     }
     else
@@ -517,11 +558,11 @@ void UIMachineMenuBar::prepareMenuHelp(QMenu *pMenu)
     {
         pMenu->addAction(gActionPool->action(UIActionIndex_Simple_WebSite));
         VBoxGlobal::connect(gActionPool->action(UIActionIndex_Simple_WebSite), SIGNAL(triggered()),
-                            &msgCenter(), SLOT(sltShowHelpWebDialog()), Qt::UniqueConnection);
+                            &msgCenter(), SLOT(sltShowHelpWebDialog()));
         fSeparator1 = true;
     }
     else
-        gActionPool->action(UIActionIndex_Simple_WebSite)->setEnabled(false);
+        gActionPool->action(RuntimeMenuHelpActionType_WebSite)->setEnabled(false);
 
     /* Separator #1: */
     if (fSeparator1)
@@ -536,7 +577,7 @@ void UIMachineMenuBar::prepareMenuHelp(QMenu *pMenu)
     {
         pMenu->addAction(gActionPool->action(UIActionIndex_Simple_ResetWarnings));
         VBoxGlobal::connect(gActionPool->action(UIActionIndex_Simple_ResetWarnings), SIGNAL(triggered()),
-                            &msgCenter(), SLOT(sltResetSuppressedMessages()), Qt::UniqueConnection);
+                            &msgCenter(), SLOT(sltResetSuppressedMessages()));
         fSeparator2 = true;
     }
     else
@@ -547,41 +588,31 @@ void UIMachineMenuBar::prepareMenuHelp(QMenu *pMenu)
         pMenu->addSeparator();
 
 
-
-#ifndef Q_WS_MAC
+#ifdef VBOX_GUI_WITH_NETWORK_MANAGER
+# ifndef Q_WS_MAC
     /* Separator #3? */
     bool fSeparator3 = false;
-#endif /* !Q_WS_MAC */
+# endif /* !Q_WS_MAC */
 
     /* Reset Warnings action: */
     if (m_pSession->allowedActionsMenuHelp() & RuntimeMenuHelpActionType_NetworkAccessManager)
     {
         pMenu->addAction(gActionPool->action(UIActionIndex_Simple_NetworkAccessManager));
         VBoxGlobal::connect(gActionPool->action(UIActionIndex_Simple_NetworkAccessManager), SIGNAL(triggered()),
-                            gNetworkManager, SLOT(show()), Qt::UniqueConnection);
-#ifndef Q_WS_MAC
-        fSeparator3 = true;
-#endif /* !Q_WS_MAC */
-    }
-    else
-        gActionPool->action(UIActionIndex_Simple_NetworkAccessManager)->setEnabled(false);
-#ifdef VBOX_WITH_REGISTRATION
-    /* Registration action: */
-    pMenu->addAction(gActionPool->action(UIActionIndex_Simple_Register));
-    VBoxGlobal::connect(gActionPool->action(UIActionIndex_Simple_Register), SIGNAL(triggered()),
-                        &vboxGlobal(), SLOT(showRegistrationDialog()), Qt::UniqueConnection);
-    VBoxGlobal::connect(gEDataEvents, SIGNAL(sigCanShowRegistrationDlg(bool)),
-                        gActionPool->action(UIActionIndex_Simple_Register), SLOT(setEnabled(bool)), Qt::UniqueConnection);
+                            gNetworkManager, SLOT(show()));
 # ifndef Q_WS_MAC
         fSeparator3 = true;
 # endif /* !Q_WS_MAC */
-#endif /* VBOX_WITH_REGISTRATION */
+    }
+    else
+        gActionPool->action(UIActionIndex_Simple_NetworkAccessManager)->setEnabled(false);
 
 # ifndef Q_WS_MAC
     /* Separator #3: */
     if (fSeparator3)
         pMenu->addSeparator();
 # endif /* !Q_WS_MAC */
+#endif /* VBOX_GUI_WITH_NETWORK_MANAGER */
 
 
     /* About action: */
