@@ -129,6 +129,12 @@ UISession::UISession(UIMachine *pMachine, CSession &sessionReference)
     , m_pMenuPool(0)
     , m_machineStatePrevious(KMachineState_Null)
     , m_machineState(session().GetMachine().GetState())
+#ifndef Q_WS_MAC
+    , m_pMachineWindowIcon(0)
+#endif /* !Q_WS_MAC */
+    , m_mouseCapturePolicy(MouseCapturePolicy_Default)
+    , m_guruMeditationHandlerType(GuruMeditationHandlerType_Default)
+    , m_hiDPIOptimizationType(HiDPIOptimizationType_None)
     , m_fIsExtensionPackUsable(false)
     , m_requestedVisualStateType(UIVisualStateType_Invalid)
 #ifdef Q_WS_WIN
@@ -142,6 +148,7 @@ UISession::UISession(UIMachine *pMachine, CSession &sessionReference)
     , m_fAllCloseActionsRestricted(false)
     , m_fSnapshotOperationsAllowed(true)
     /* Common flags: */
+    , m_fIsStarted(false)
     , m_fIsFirstTimeStarted(false)
     , m_fIsIgnoreRuntimeMediumsChanging(false)
     , m_fIsGuestResizeIgnored(false)
@@ -269,8 +276,8 @@ void UISession::powerUp()
     if (isSaved())
     {
         msgCenter().showModalProgressDialog(progress, machine.GetName(), ":/progress_state_restore_90px.png", 0, 0);
-        /* After restoring from 'saved' state, guest screen size should be adjusted: */
-        machineLogic()->maybeAdjustGuestScreenSize();
+        /* After restoring from 'saved' state, machine-window(s) geometry should be adjusted: */
+        machineLogic()->adjustMachineWindowsGeometry();
     }
     else
         msgCenter().showModalProgressDialog(progress, machine.GetName(), ":/progress_start_90px.png");
@@ -364,7 +371,7 @@ void UISession::powerUp()
 #endif
 
     /* Warn listeners about machine was started: */
-    emit sigMachineStarted();
+    emit sigStarted();
 }
 
 bool UISession::saveState()
@@ -914,7 +921,7 @@ void UISession::sltHandleHostScreenCountChange()
     LogRelFlow(("UISession: Host-screen count changed.\n"));
 
     /* Notify current machine-logic: */
-    emit sigHostScreenCountChanged();
+    emit sigHostScreenCountChange();
 }
 
 void UISession::sltHandleHostScreenGeometryChange()
@@ -922,7 +929,15 @@ void UISession::sltHandleHostScreenGeometryChange()
     LogRelFlow(("UISession: Host-screen geometry changed.\n"));
 
     /* Notify current machine-logic: */
-    emit sigHostScreenGeometryChanged();
+    emit sigHostScreenGeometryChange();
+}
+
+void UISession::sltHandleHostScreenAvailableAreaChange()
+{
+    LogRelFlow(("UISession: Host-screen available-area changed.\n"));
+
+    /* Notify current machine-logic: */
+    emit sigHostScreenAvailableAreaChange();
 }
 
 void UISession::sltAdditionsChange()
@@ -1011,6 +1026,7 @@ void UISession::prepareConsoleEventHandlers()
 
 void UISession::prepareConnections()
 {
+    connect(this, SIGNAL(sigStarted()), this, SLOT(sltMarkStarted()));
     connect(this, SIGNAL(sigCloseRuntimeUI()), this, SLOT(sltCloseRuntimeUI()));
 
 #ifdef Q_WS_MAC
@@ -1023,7 +1039,7 @@ void UISession::prepareConnections()
     connect(QApplication::desktop(), SIGNAL(resized(int)),
             this, SLOT(sltHandleHostScreenGeometryChange()));
     connect(QApplication::desktop(), SIGNAL(workAreaResized(int)),
-            this, SLOT(sltHandleHostScreenGeometryChange()));
+            this, SLOT(sltHandleHostScreenAvailableAreaChange()));
 #endif /* !Q_WS_MAC */
 }
 
@@ -1116,6 +1132,28 @@ void UISession::loadSessionSettings()
         /* Temporary: */
         QString strSettings;
 
+#ifndef Q_WS_MAC
+        /* Load/prepare user's machine-window icon: */
+        QIcon icon;
+        foreach (const QString &strIconName, VBoxGlobal::machineWindowIconNames(machine))
+            if (!strIconName.isEmpty())
+                icon.addFile(strIconName);
+        if (!icon.isNull())
+            m_pMachineWindowIcon = new QIcon(icon);
+
+        /* Load user's machine-window name postfix: */
+        m_strMachineWindowNamePostfix = VBoxGlobal::machineWindowNamePostfix(machine);
+#endif /* !Q_WS_MAC */
+
+        /* Determine mouse-capture policy: */
+        m_mouseCapturePolicy = VBoxGlobal::mouseCapturePolicy(machine);
+
+        /* Determine Guru Meditation handler type: */
+        m_guruMeditationHandlerType = VBoxGlobal::guruMeditationHandlerType(machine);
+
+        /* Determine HiDPI optimization type: */
+        m_hiDPIOptimizationType = VBoxGlobal::hiDPIOptimizationType(machine);
+
         /* Is there should be First RUN Wizard? */
         strSettings = machine.GetExtraData(GUI_FirstRun);
         if (strSettings == "yes")
@@ -1178,6 +1216,12 @@ void UISession::saveSessionSettings()
         SystemParametersInfo(SPI_SETSCREENSAVEACTIVE, true, 0, 0);
 # endif /* Q_WS_WIN */
 #endif
+
+#ifndef Q_WS_MAC
+        /* Cleanup user's machine-window icon: */
+        delete m_pMachineWindowIcon;
+        m_pMachineWindowIcon = 0;
+#endif /* !Q_WS_MAC */
     }
 }
 
