@@ -112,12 +112,17 @@ def_uninstall()
 
     . ./deffiles
     found=0
+    for i in "/opt/$PACKAGE-"*; do
+        test -e "$i" && found=1
+    done
     for i in $DEFAULT_FILE_NAMES; do
-        test "$found" = 0 -a -e "$i" && found=1
+        test "$found" = 0 && test -e "$i" && found=1
     done
     test "$found" = 0 &&
-        for i in $DEFAULT_VERSIONED_FILE_NAMES-*; do
-            test "$found" = 0 -a -e "$i" && found=1
+        for i in $DEFAULT_VERSIONED_FILE_NAMES; do
+            for j in $i-*; do
+                test "$found" = 0 && test -e "$j" && found=1
+            done
         done
     test "$found" = 0 && return 0
     if ! test "$1" = "force" ; then
@@ -157,7 +162,8 @@ EOF
     for i in $DEFAULT_VERSIONED_FILE_NAMES; do
         rm -f "$i-"* 2> /dev/null
     done
-    rm -f "/usr/lib/$PACKAGE" "/usr/lib64/$PACKAGE" "/usr/share/$PACKAGE"
+    rm -f "/usr/lib/$PACKAGE" "/usr/lib64/$PACKAGE" "/usr/share/$PACKAGE" \
+        "/usr/lib/i386-linux-gnu/$PACKAGE" "/usr/lib/x86_64-linux-gnu/$PACKAGE"
 
     # And any packages left under /opt
     for i in "/opt/$PACKAGE-"*; do
@@ -175,15 +181,11 @@ cpu=`uname -m`;
 case "$cpu" in
   i[3456789]86|x86)
     cpu="x86"
-    lib_path="/usr/lib"
+    lib_candidates="/usr/lib/i386-linux-gnu /usr/lib /lib"
     ;;
   x86_64|amd64)
     cpu="amd64"
-    if test -d "/usr/lib64"; then
-      lib_path="/usr/lib64"
-    else
-      lib_path="/usr/lib"
-    fi
+    lib_candidates="/usr/lib/x86_64-linux-gnu /usr/lib64 /usr/lib /lib64 /lib"
     ;;
   *)
     cpu="unknown"
@@ -193,6 +195,20 @@ if [ ! -r "$ARCH_PACKAGE" ]; then
   info "Detected unsupported $cpu machine type."
   exit 1
 fi
+# Find the most appropriate libary folder by seeing which of the candidate paths
+# are actually in the shared linker path list and choosing the first.  We look
+# for Debian-specific paths first, then LSB ones, then the new RedHat ones.
+libs=`ldconfig -v 2>/dev/null | grep -v ^$'\t'`
+for i in $lib_candidates; do
+  if echo $libs | grep -q $i; then
+    lib_path=$i
+    break
+  fi
+done
+if [ ! -x "$lib_path" ]; then
+  info "Unable to determine correct library path."
+  exit 1
+fi
 
 # Sensible default actions
 ACTION="install"
@@ -312,7 +328,7 @@ if [ -d installer ]; then
           info "Error: Failed to copy installer module \"$CUR_FILE\""
           if ! test "$FORCE_UPGRADE" = "force"; then
               exit 1
-          fi        
+          fi
       fi
   done
 fi
@@ -342,6 +358,7 @@ bzip2 -d -c "$ARCH_PACKAGE" | tar -xf - -C "$INSTALLATION_DIR" || exit 1
 link_into_fs "bin" "/usr/bin"
 link_into_fs "sbin" "/usr/sbin"
 link_into_fs "lib" "$lib_path"
+add_symlink "$INSTALLATION_DIR/lib/$PACKAGE" /usr/lib/"$PACKAGE"
 link_into_fs "share" "/usr/share"
 link_into_fs "src" "/usr/src"
 
@@ -360,7 +377,7 @@ do
         info "Error: Failed to install module \"$CUR_MODULE\""
         if ! test "$FORCE_UPGRADE" = "force"; then
             exit 1
-        fi        
+        fi
     fi
 done
 
